package com.btcpiyush.ads.google_applovin_unity_ads.open_ad

import android.app.Activity
import android.util.Log
import com.btcpiyush.ads.google_applovin_unity_ads.CustomOpenAdDialog
import com.btcpiyush.ads.google_applovin_unity_ads.GoogleApplovinUnityAdsPlugin
import com.btcpiyush.ads.google_applovin_unity_ads.R
import com.btcpiyush.ads.google_applovin_unity_ads.api.AppSettings
import com.btcpiyush.ads.google_applovin_unity_ads.api.GoogleAdsSetting
import com.google.android.gms.ads.AdError
import com.google.android.gms.ads.AdRequest
import com.google.android.gms.ads.FullScreenContentCallback
import com.google.android.gms.ads.LoadAdError
import com.google.android.gms.ads.appopen.AppOpenAd
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import java.util.*

class AdmobOpenAd(
    private val flutterPluginBinding: FlutterPlugin.FlutterPluginBinding,
    private val act: Activity,
    private val settingResponse: AppSettings
) :
    MethodChannel.MethodCallHandler {

    companion object {
        val allAds: MutableMap<Int, OpenAdModel> = mutableMapOf()
        val TAG: String = "ADOPEN ADS"
    }

    private fun getGoogleAdRequest(): AdRequest {
        return AdRequest.Builder().build()
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        when (call.method) {
            "call.ads.openad.isLoaded" -> {
                val id = call.argument<Int>("id")
                val model = allAds[id] ?: return result.success(false)

                if (model.isLoaded)
                    result.success(true)
                else
                    result.success(false)
            }

            "call.ads.openad" -> {
                val id: Int = call.argument<Int?>("id") ?: 0

                showLog("id  ======  $id")

                if (allAds[id] == null) {
                    allAds[id] = OpenAdModel()
                }

                val model = allAds[id]

                model?.let { model ->
                    model.pos = -1
                    loadOpenAds(id, model)
                }
            }

            "call.ads.openad.show" -> {
                val id: Int = call.argument<Int>("id") ?: 0
                val model = allAds[id]

                model?.let { model ->
                    showOpenAds(id, model)
                }
            }

            "call.ads.openad.dispose" -> {
                val id: Int = call.argument<Int>("id") ?: 0
                allAds.remove(id)
            }
            else -> result.notImplemented()
        }
    }

    private fun loadOpenAds(id: Int, model: OpenAdModel) {
        val pos = ++model.pos
        showLog("pos ======== $pos")
        val adChannel = MethodChannel(
            flutterPluginBinding.binaryMessenger,
            "${GoogleApplovinUnityAdsPlugin.CHANNEL_OPENAD_NAME}_$id"
        )

        if (settingResponse.adsSequence.size > 0 && settingResponse.adsSequence.size > pos) {
            when (settingResponse.adsSequence[pos].lowercase()) {
                act.getString(R.string.google_ads).lowercase() -> {
                    settingResponse.googleAds?.let { google_ads ->
                        if (google_ads.isOpenAds == true) {
                            loadGoogleOpenAds(
                                id,
                                model,
                                google_ads
                            )
                            return@loadOpenAds
                        }
                    }
                }

                act.getString(R.string.custom_ads).lowercase() -> {
                    settingResponse.customAds?.let { customAds ->
                        if (customAds.isOpenAds == true) {

                            if (model.customOpenAd != null) {
                                showLog("= ≠= =≠=≠≠=≠≠≠≠≠≠≠")
                                adChannel?.invokeMethod(
                                    GoogleApplovinUnityAdsPlugin.METHOD_SUCCESS,
                                    null
                                )
                                return
                            }

                            model.isLoaded = false

                            model.customOpenAd = CustomOpenAdDialog(
                                act,
                                customAds,
                                settingResponse.appTitle,
                                settingResponse.appLogo,
                                object : CustomOpenAdDialog.DialogInterface {
                                    override fun onLoaded() {
                                        showLog("¢¢¢¢¢¢¢¢¢¢¢¢¢¢¢¢")
                                        model.isLoaded = true
                                        if (model.isDefaultShowing) {
                                            showCustomOpenAds(id, model)
                                        } else {
                                            adChannel?.invokeMethod(
                                                GoogleApplovinUnityAdsPlugin.METHOD_SUCCESS,
                                                null
                                            )
                                        }
                                    }

                                    override fun onDismiss() {
                                        showLog("= ≠= =≠=≠≠=¢¢¢¢¢¢¢¢¢¢¢¢¢")
                                        model.isLoaded = false
                                        model.customOpenAd = null
                                        model.isLoaded = false
                                        model.isDefaultShowing = false
                                        model.pos = -1
                                        loadOpenAds(id, model)
                                        adChannel?.invokeMethod(
                                            GoogleApplovinUnityAdsPlugin.METHOD_SUCCESS,
                                            null
                                        )
                                    }

                                    override fun onFailed() {
                                        showLog("= ≠= =≠=≠≠=≠≠¢¢¢¢¢¢¢¢¢¢¢¢¢")
                                        model.isLoaded = false
                                        model.customOpenAd = null
                                        loadOpenAds(id, model)
                                    }
                                })

                            return@loadOpenAds
                        }
                    }
                }
            }
            loadOpenAds(id, model)
        } else {
            model.isLoaded = false
            adChannel?.invokeMethod(GoogleApplovinUnityAdsPlugin.METHOD_ERROR, null)
        }
    }

    private fun loadGoogleOpenAds(
        id: Int,
        model: OpenAdModel,
        it: GoogleAdsSetting
    ) {
        val adChannel = MethodChannel(
            flutterPluginBinding.binaryMessenger,
            "${GoogleApplovinUnityAdsPlugin.CHANNEL_OPENAD_NAME}_$id"
        )


        if (model.openAd != null) {
            adChannel?.invokeMethod(GoogleApplovinUnityAdsPlugin.METHOD_SUCCESS, null)
            /*if(model.isLoaded)
                showOpenAds(id, model)*/
            return@loadGoogleOpenAds
        }

        val admobID = it.appOpenId ?: ""

        if (!admobID.isNullOrEmpty()) {
            model.isLoaded = false

            showLog("$admobID is not empty ")

            val callback = object : AppOpenAd.AppOpenAdLoadCallback() {
                override fun onAdLoaded(ad: AppOpenAd) {
                    model.isLoaded = true
                    model.openAd = ad
                    if (model.isDefaultShowing) {
                        showGoogleOpenAds(id, model)
                    } else {
                        adChannel?.invokeMethod(GoogleApplovinUnityAdsPlugin.METHOD_SUCCESS, null)
                    }
                }

                override fun onAdFailedToLoad(loadAdError: LoadAdError) {
                    showLog(
                        " -------- onAdFailedToLoad OpenAd  --- $loadAdError"
                    )
                    loadOpenAds(id, model)
                }
            }

            AppOpenAd.load(
                act,
                admobID,
                getGoogleAdRequest(),
                AppOpenAd.APP_OPEN_AD_ORIENTATION_PORTRAIT,
                callback
            )
            return@loadGoogleOpenAds
        }

        loadOpenAds(id, model)
    }


    private fun showOpenAds(
        id: Int,
        model: OpenAdModel
    ) {
        val pos = model.pos
        showLog("= ≠= =≠=≠≠=≠≠≠≠≠≠≠≠≠≠≠≠≠=== === = ==  ,mvn,mcbvc,b,cvb, c,v=≠≠≠≠≠≠≠≠≠≠≠≠  = = == = = = ≠≠≠≠≠≠≠≠   ¢¢¢¢¢¢¢¢¢¢¢¢¢¢¢¢     $pos")
        if (settingResponse.adsSequence.size > 0 && settingResponse.adsSequence.size > pos) {
            when (settingResponse.adsSequence[pos].lowercase()) {
                act.getString(R.string.google_ads).lowercase() -> {
                    showLog("= ≠= =≠=≠≠≠≠≠≠   ¢¢¢¢¢¢¢¢¢¢¢¢¢¢¢¢")
                    showGoogleOpenAds(id, model)
                    //return@showOpenAds
                }

                act.getString(R.string.custom_ads).lowercase() -> {
                    showLog("= ≠= =≠=≠≠=≠≠≠≠≠≠≠≠≠≠≠≠≠=== === = ==  ¢¢¢¢¢¢¢¢¢¢¢¢")
                    showCustomOpenAds(id, model)
                    //return@showOpenAds
                }
            }
          //  showOpenAds(id, model)
        } else {
            showLog("= ≠= =≠=≠≠=≠≠≠≠≠≠≠≠≠≠≠≠≠=== === = = ¢¢¢¢¢¢¢¢¢¢¢¢¢¢¢¢     $pos")
            model.isDefaultShowing = true
            model.pos = -1
            loadOpenAds(id, model)
        }
    }

    private fun showGoogleOpenAds(
        id: Int,
        model: OpenAdModel
    ) {
        model.openAd?.let {
            try {
                val adChannel = MethodChannel(
                    flutterPluginBinding.binaryMessenger,
                    "${GoogleApplovinUnityAdsPlugin.CHANNEL_OPENAD_NAME}_$id"
                )

                it.fullScreenContentCallback = object : FullScreenContentCallback() {
                    /** Called when full screen content is dismissed.  */
                    override fun onAdDismissedFullScreenContent() {
                        // Set the reference to null so isAdAvailable() returns false.

                        showLog("Open AD: :  onAdDismissedFullScreenContent   Open Ad Request")
                        model.openAd = null
                        model.isLoaded = false
                        model.isDefaultShowing = false
                        model.pos = -1
                        loadOpenAds(id, model)
                        adChannel?.invokeMethod(GoogleApplovinUnityAdsPlugin.METHOD_SUCCESS, null)
                    }

                    override fun onAdFailedToShowFullScreenContent(adError: AdError) {
                        super.onAdFailedToShowFullScreenContent(adError)
                        showLog(
                            "ADOpen onAdFailedToShowFullScreenContent"
                        )
                        Log.e("Open AD: ", "onAdFailedToShowFullScreenContent Open Ad Request")
                        model.openAd = null
                        model.isLoaded = false
                        model.isDefaultShowing = false
                        model.isDefaultShowing = true
                        loadOpenAds(id, model)
                    }

                    /** Called when fullscreen content is shown.  */
                    override fun onAdShowedFullScreenContent() {
                        showLog(
                            "open_ad ADOpen onAdShowedFullScreenContent"
                        )
                    }
                }
                it.show(act)
                return@showGoogleOpenAds
            } catch (e: Exception) {
                e.printStackTrace()
                model.isDefaultShowing = true
            }
        }
        model.pos++
        showOpenAds(id, model)
    }

    private fun showCustomOpenAds(
        id: Int,
        model: OpenAdModel
    ) {
        model.customOpenAd?.let {
            it.show()
            it.setStartTimer()
            return@showCustomOpenAds
        }
        model.pos++
        showOpenAds(id, model)
    }

    private fun showLog(error: String) {
        Log.e(TAG, error)
    }

}