import 'dart:async';
import 'dart:io';

import 'package:flutter/services.dart';
import 'package:google_applovin_unity_ads/events/admob_events.dart';
import 'package:google_applovin_unity_ads/events/event_handler.dart';

class OpenAd extends AdmobEventHandler {

  static const String _channelName = 'dev.bitcoding.ads/open_ad';
  static const String channelMethod = "call.ads.openad";
  static const MethodChannel channel =
      MethodChannel(_channelName);
  late int id;
  late MethodChannel _adChannel;
  final void Function(AdmobAdEvent, Map<String, dynamic>?)? listener;

  OpenAd({
    this.listener,
  }) : super(listener) {
    id = hashCode;
    if (listener != null) {
      _adChannel = MethodChannel('${_channelName}_$id');
      _adChannel.setMethodCallHandler(handleEvent);
    }
  }

  Future<bool?> get isLoaded async {
    final result = await channel.invokeMethod('$channelMethod.isLoaded', _channelMethodsArguments);
    return result;
  }

  void load() async {
    await channel.invokeMethod(channelMethod,
      _channelMethodsArguments
    );
  }

  void show() async {
    await channel.invokeMethod('$channelMethod.show',
        _channelMethodsArguments
    );
  }

  void dispose() async {
    await channel.invokeMethod('$channelMethod.dispose', _channelMethodsArguments);
  }

  Map<String, dynamic> get _channelMethodsArguments => <String, dynamic>{
    'id': id,
  };
}
